package ru.pricepulse.parsingservice.ozon_parser.service.parsing;

import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.springframework.context.annotation.Profile;
import org.springframework.retry.annotation.Recover;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import ru.pricepulse.parsingservice.enumeration.Category;
import ru.pricepulse.parsingservice.enumeration.Marketplace;
import ru.pricepulse.parsingservice.ozon_parser.pool.WebDriverPool;
import ru.pricepulse.parsingservice.ozon_parser.service.dto.ParsedData;
import ru.pricepulse.parsingservice.ozon_parser.service.page.AccessDeniedPage;
import ru.pricepulse.parsingservice.ozon_parser.service.page.CategoryPage;
import ru.pricepulse.parsingservice.ozon_parser.service.page.NoContentPage;
import ru.pricepulse.parsingservice.service.ProductService;

@Slf4j
@Service
@Profile("ozon")
public class OzonCategoryPageParsingService {

    private final WebDriverPool webDriverPool;

    private final ProductService productService;

    public OzonCategoryPageParsingService(WebDriverPool webDriverPool,
                                          ProductService productService) {
        this.webDriverPool = webDriverPool;
        this.productService = productService;
    }

    @Retryable(maxAttempts = 10, recover = "recover")
    public void parseCategoryPage(String pageUrl, Category category, AtomicBoolean stopFlag) {
        var driver = webDriverPool.borrowDriver();
        if (driver == null) {
            throw new RuntimeException();
        }
        try {
            driver.manage().timeouts().pageLoadTimeout(Duration.of(10, ChronoUnit.SECONDS));
            driver.get(pageUrl);
            WebDriverWait wait = new WebDriverWait(driver, Duration.of(10, ChronoUnit.SECONDS));
            var accessDeniedPage = new AccessDeniedPage(driver, wait);
            var categoryPage = new CategoryPage(driver, wait);
            var noContentPage = new NoContentPage(driver, wait);
            wait.until(d -> checkForWaitingPageLoading(accessDeniedPage, categoryPage));
            if (checkAccessDeniedPage(accessDeniedPage)) {
                log.info("Доступ ограничен, пробуем решить проблему: {}", pageUrl);
                resolveAccessDeniedPage(accessDeniedPage);
                log.info("Проблема успешно решена: {}", pageUrl);
            }
            if (noContentPage.isLoaded()) {
                log.info("Страница не найдена");
                stopFlag.set(true);
                return;
            }

            log.info("Получаем список товаров на текущей странице: {}", pageUrl);
            List<ParsedData> parsedData;
            try {
                parsedData = categoryPage.getParsedProducts();
                for (ParsedData data : parsedData) {
                    data.setMarketplace(Marketplace.OZON);
                    data.setCategory(category);
                }
                productService.saveBatch(parsedData);
            } catch (Exception e) {
                throw new Exception(e);
            }
            webDriverPool.returnDriver(driver);
        } catch (Exception ignored) {
            throw new RuntimeException(ignored);
        } finally {
            webDriverPool.returnDriver(driver);
        }
    }

    private boolean checkForWaitingPageLoading(AccessDeniedPage accessDeniedPage,
                                               CategoryPage categoryPage) {
        log.debug("Проверка что страница 'Доступ ограничен'");
        try {
            if (checkAccessDeniedPage(accessDeniedPage)) {
                return true;
            }
        } catch (Exception e) {
            log.debug("Ошибка проверки", e);
        }
        log.debug("Проверка что страница 'Страница категории'");
        if (checkCategoryPage(categoryPage)) {
            return true;
        }
        log.debug("Проверка загрузки страницы неудачна");
        return false;
    }

    private boolean checkAccessDeniedPage(AccessDeniedPage accessDeniedPage) {
        return accessDeniedPage.isLoaded();
    }

    private boolean checkCategoryPage(CategoryPage categoryPage) {
        return categoryPage.isLoaded();
    }

    private void resolveAccessDeniedPage(AccessDeniedPage accessDeniedPage) {
        accessDeniedPage.clickReloadButton();
    }

    @Recover
    private void recover(Exception e) {
        log.error("Все ретраи провалились");
    }

}
